@@ -263,9 +263,19 @@ void ConnectionIcon::setStates()
 
         if (!vpnConnection) {
 #if NM_CHECK_VERSION(0, 9, 10)
-            if (activeConnection->state() == NetworkManager::ActiveConnection::Activating && activeConnection->type() != NetworkManager::ConnectionSettings::Generic) {
+            if (activeConnection->state() == NetworkManager::ActiveConnection::Activating &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Bond &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Bridge &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Generic &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Infiniband &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Team &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Vlan) {
 #else
-            if (activeConnection->state() == NetworkManager::ActiveConnection::Activating) {
+            if (activeConnection->state() == NetworkManager::ActiveConnection::Activating &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Bond &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Bridge &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Infiniband &&
+                activeConnection->type() != NetworkManager::ConnectionSettings::Vlan) {
 #endif
                 connecting = true;
             }
