@@ -1,5 +1,6 @@
 /*
     Copyright 2013 Lukas Tinkl <ltinkl@redhat.com>
+    Copyright 2015 Jan Grulich <jgrulich@redhat.com>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -34,7 +35,7 @@
 class OpenVpnAdvancedWidget::Private {
 public:
     NetworkManager::VpnSetting::Ptr setting;
-    KProcess * openvpnProcess;
+    KProcess *openvpnProcess;
     QByteArray openVpnCiphers;
     bool gotOpenVpnCiphers;
     bool readConfig;
@@ -65,8 +66,9 @@ OpenVpnAdvancedWidget::OpenVpnAdvancedWidget(const NetworkManager::VpnSetting::P
     d->readConfig = false;
     d->setting = setting;
 
-    connect(m_ui->proxyPasswordStorage, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &OpenVpnAdvancedWidget::proxyPasswordStorageChanged);
-    connect(m_ui->chkProxyShowPassword, &QCheckBox::toggled, this, &OpenVpnAdvancedWidget::proxyPasswordToggled);
+    m_ui->proxyPassword->setPasswordOptionsEnabled(true);
+    m_ui->proxyPassword->setPasswordOptionEnabled(PasswordField::NotRequired, true);
+
     connect(m_ui->cmbProxyType, static_cast<void (QComboBox::*)(int)>(&QComboBox::currentIndexChanged), this, &OpenVpnAdvancedWidget::proxyTypeChanged);
 
     // start openVPN process and get its cipher list
@@ -86,7 +88,9 @@ OpenVpnAdvancedWidget::OpenVpnAdvancedWidget(const NetworkManager::VpnSetting::P
 
     KAcceleratorManager::manage(this);
 
-    loadConfig();
+    if (d->setting && !d->setting->isNull()) {
+        loadConfig();
+    }
 }
 
 OpenVpnAdvancedWidget::~OpenVpnAdvancedWidget()
@@ -152,34 +156,45 @@ void OpenVpnAdvancedWidget::loadConfig()
     const NMStringMap secrets = d->setting->secrets();
 
     // Optional Settings
-    if (dataMap.contains(NM_OPENVPN_KEY_PORT)) {
-        m_ui->sbCustomPort->setValue(dataMap[NM_OPENVPN_KEY_PORT].toUInt());
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_PORT))) {
+        m_ui->sbCustomPort->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_PORT)].toUInt());
     } else {
         m_ui->sbCustomPort->setValue(0);
     }
-    if (dataMap.contains(NM_OPENVPN_KEY_TUNNEL_MTU)) {
-        m_ui->sbMtu->setValue(dataMap[NM_OPENVPN_KEY_TUNNEL_MTU].toUInt());
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_TUNNEL_MTU))) {
+        m_ui->sbMtu->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_TUNNEL_MTU)].toUInt());
     } else {
         m_ui->sbMtu->setValue(0);
     }
-    if (dataMap.contains(NM_OPENVPN_KEY_FRAGMENT_SIZE)) {
-        m_ui->sbUdpFragmentSize->setValue(dataMap[NM_OPENVPN_KEY_FRAGMENT_SIZE].toUInt());
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_FRAGMENT_SIZE))) {
+        m_ui->sbUdpFragmentSize->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_FRAGMENT_SIZE)].toUInt());
     } else {
         m_ui->sbUdpFragmentSize->setValue(0);
     }
-    if (dataMap.contains(NM_OPENVPN_KEY_RENEG_SECONDS)) {
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_RENEG_SECONDS))) {
         m_ui->chkUseCustomReneg->setChecked(true);
-        m_ui->sbCustomReneg->setValue(dataMap[NM_OPENVPN_KEY_RENEG_SECONDS].toUInt());
+        m_ui->sbCustomReneg->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_RENEG_SECONDS)].toUInt());
     } else {
         m_ui->chkUseCustomReneg->setChecked(false);
         m_ui->sbCustomReneg->setValue(0);
     }
-    m_ui->chkUseLZO->setChecked( dataMap[NM_OPENVPN_KEY_COMP_LZO] == "yes" );
-    m_ui->chkUseTCP->setChecked( dataMap[NM_OPENVPN_KEY_PROTO_TCP] == "yes" );
-    m_ui->chkUseTAP->setChecked( dataMap[NM_OPENVPN_KEY_TAP_DEV] == "yes" );
-    m_ui->chkMssRestrict->setChecked(dataMap[NM_OPENVPN_KEY_MSSFIX] == "yes");
+    m_ui->chkUseLZO->setChecked(dataMap[QLatin1String(NM_OPENVPN_KEY_COMP_LZO)] == QLatin1String("yes"));
+    m_ui->chkUseTCP->setChecked(dataMap[QLatin1String(NM_OPENVPN_KEY_PROTO_TCP)] == QLatin1String("yes"));
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_DEV_TYPE))) {
+        m_ui->chkUseVirtualDeviceType->setChecked(true);
+        if (dataMap[QLatin1String(NM_OPENVPN_KEY_DEV_TYPE)] == QLatin1String("tap")) {
+            m_ui->cmbDeviceType->setCurrentIndex(1);
+        }
+    }
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_DEV))) {
+        m_ui->chkUseVirtualDeviceName->setChecked(true);
+        m_ui->leVirtualDeviceName->setText(dataMap[QLatin1String(NM_OPENVPN_KEY_DEV)]);
+    }
+    m_ui->chkMssRestrict->setChecked(dataMap[QLatin1String(NM_OPENVPN_KEY_MSSFIX)] == QLatin1String("yes"));
+    m_ui->chkRandRemHosts->setChecked(dataMap[QLatin1String(NM_OPENVPN_KEY_REMOTE_RANDOM)] == QLatin1String("yes"));
+
     // Optional Security Settings
-    QString hmacKeyAuth = dataMap[NM_OPENVPN_KEY_AUTH];
+    const QString hmacKeyAuth = dataMap[QLatin1String(NM_OPENVPN_KEY_AUTH)];
     if (hmacKeyAuth == QLatin1String(NM_OPENVPN_AUTH_NONE)) {
         m_ui->cboHmac->setCurrentIndex(Private::EnumHashingAlgorithms::None);
     } else if (hmacKeyAuth == QLatin1String(NM_OPENVPN_AUTH_RSA_MD4)) {
@@ -201,71 +216,74 @@ void OpenVpnAdvancedWidget::loadConfig()
     } else {
         m_ui->cboHmac->setCurrentIndex(Private::EnumHashingAlgorithms::Default);
     }
+
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_KEYSIZE))) {
+        m_ui->chkUseCustomCipherKey->setChecked(true);
+        m_ui->sbCustomCipherKey->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_KEYSIZE)].toUInt());
+    }
+
     // ciphers populated above?
-    if (d->gotOpenVpnCiphers && dataMap.contains(NM_OPENVPN_KEY_CIPHER)) {
-        m_ui->cboCipher->setCurrentIndex(m_ui->cboCipher->findText(dataMap[NM_OPENVPN_KEY_CIPHER]));
+    if (d->gotOpenVpnCiphers && dataMap.contains(QLatin1String(NM_OPENVPN_KEY_CIPHER))) {
+        m_ui->cboCipher->setCurrentIndex(m_ui->cboCipher->findText(dataMap[QLatin1String(NM_OPENVPN_KEY_CIPHER)]));
     }
 
     // Optional TLS
-    if (dataMap.contains(NM_OPENVPN_KEY_TLS_REMOTE)) {
-        m_ui->subjectMatch->setText(dataMap[NM_OPENVPN_KEY_TLS_REMOTE]);
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_TLS_REMOTE))) {
+        m_ui->subjectMatch->setText(dataMap[QLatin1String(NM_OPENVPN_KEY_TLS_REMOTE)]);
     }
 
-    if (dataMap.contains(NM_OPENVPN_KEY_REMOTE_CERT_TLS)) {
-        const QString remoteCertTls = dataMap[NM_OPENVPN_KEY_REMOTE_CERT_TLS];
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_REMOTE_CERT_TLS))) {
+        const QString remoteCertTls = dataMap[QLatin1String(NM_OPENVPN_KEY_REMOTE_CERT_TLS)];
         m_ui->chkRemoteCertTls->setChecked(true);
         m_ui->labelRemoteCertTls->setEnabled(true);
         m_ui->cmbRemoteCertTls->setEnabled(true);
         m_ui->cmbRemoteCertTls->setCurrentIndex(remoteCertTls == QLatin1String("server") ? 0 : 1);
     }
 
-    m_ui->useExtraTlsAuth->setChecked(!dataMap[NM_OPENVPN_KEY_TA].isEmpty());
-    m_ui->kurlTlsAuthKey->setUrl(QUrl::fromLocalFile(dataMap[NM_OPENVPN_KEY_TA]) );
-    if (dataMap.contains(NM_OPENVPN_KEY_TA_DIR)) {
-        const uint tlsAuthDirection = dataMap[NM_OPENVPN_KEY_TA_DIR].toUInt();
+    m_ui->useExtraTlsAuth->setChecked(!dataMap[QLatin1String(NM_OPENVPN_KEY_TA)].isEmpty());
+    m_ui->kurlTlsAuthKey->setUrl(QUrl::fromLocalFile(dataMap[QLatin1String(NM_OPENVPN_KEY_TA)]));
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_TA_DIR))) {
+        const uint tlsAuthDirection = dataMap[QLatin1String(NM_OPENVPN_KEY_TA_DIR)].toUInt();
         m_ui->cboDirection->setCurrentIndex(tlsAuthDirection + 1);
     }
 
     // Proxies
-    if (dataMap[NM_OPENVPN_KEY_PROXY_TYPE] == "http") {
+    if (dataMap[QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE)] == QLatin1String("http")) {
         m_ui->cmbProxyType->setCurrentIndex(Private::EnumProxyType::HTTP);
-    } else if (dataMap[NM_OPENVPN_KEY_PROXY_TYPE] == "socks") {
+    } else if (dataMap[QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE)] == QLatin1String("socks")) {
         m_ui->cmbProxyType->setCurrentIndex(Private::EnumProxyType::SOCKS);
     } else {
         m_ui->cmbProxyType->setCurrentIndex(Private::EnumProxyType::NotRequired);
     }
     proxyTypeChanged(m_ui->cmbProxyType->currentIndex());
-    m_ui->proxyServerAddress->setText(dataMap[NM_OPENVPN_KEY_PROXY_SERVER]);
-    if (dataMap.contains(NM_OPENVPN_KEY_PROXY_PORT)) {
-        m_ui->sbProxyPort->setValue(dataMap[NM_OPENVPN_KEY_PROXY_PORT].toUInt());
+    m_ui->proxyServerAddress->setText(dataMap[QLatin1String(NM_OPENVPN_KEY_PROXY_SERVER)]);
+    if (dataMap.contains(QLatin1String(NM_OPENVPN_KEY_PROXY_PORT))) {
+        m_ui->sbProxyPort->setValue(dataMap[QLatin1String(NM_OPENVPN_KEY_PROXY_PORT)].toUInt());
     } else {
         m_ui->sbProxyPort->setValue(0);
     }
-    m_ui->chkProxyRetry->setChecked(dataMap[NM_OPENVPN_KEY_PROXY_RETRY] == "yes");
-    m_ui->proxyUsername->setText(dataMap[NM_OPENVPN_KEY_HTTP_PROXY_USERNAME]);
+    m_ui->chkProxyRetry->setChecked(dataMap[QLatin1String(NM_OPENVPN_KEY_PROXY_RETRY)] == QLatin1String("yes"));
+    m_ui->proxyUsername->setText(dataMap[QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_USERNAME)]);
     d->readConfig = true;
 
     NetworkManager::Setting::SecretFlags type;
     type = (NetworkManager::Setting::SecretFlags)dataMap[NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"].toInt();
     if (!(type & NetworkManager::Setting::NotSaved || type & NetworkManager::Setting::NotRequired)) {
-        m_ui->proxyPassword->setText(secrets.value(NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD));
+        m_ui->proxyPassword->setText(secrets.value(QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD)));
     }
-    fillOnePasswordCombo(m_ui->proxyPasswordStorage, type);
+    fillOnePasswordCombo(m_ui->proxyPassword, type);
 }
 
-void OpenVpnAdvancedWidget::setPasswordType(QLineEdit *edit, int type)
+void OpenVpnAdvancedWidget::fillOnePasswordCombo(PasswordField *passwordField, NetworkManager::Setting::SecretFlags type)
 {
-    edit->setEnabled(type == SettingWidget::EnumPasswordStorageType::Store);
-}
-
-void OpenVpnAdvancedWidget::fillOnePasswordCombo(QComboBox * combo, NetworkManager::Setting::SecretFlags type)
-{
-    if (type.testFlag(NetworkManager::Setting::AgentOwned) || type.testFlag(NetworkManager::Setting::None)) {
-        combo->setCurrentIndex(SettingWidget::EnumPasswordStorageType::Store);
-    } else if (type.testFlag(NetworkManager::Setting::NotRequired)) {
-        combo->setCurrentIndex(SettingWidget::EnumPasswordStorageType::NotRequired);
+    if (type.testFlag(NetworkManager::Setting::None)) {
+        passwordField->setPasswordOption(PasswordField::StoreForAllUsers);
+    } else if (type.testFlag(NetworkManager::Setting::AgentOwned)) {
+        passwordField->setPasswordOption(PasswordField::StoreForUser);
     } else if (type.testFlag(NetworkManager::Setting::NotSaved)) {
-        combo->setCurrentIndex(SettingWidget::EnumPasswordStorageType::AlwaysAsk);
+        passwordField->setPasswordOption(PasswordField::AlwaysAsk);
+    } else {
+        passwordField->setPasswordOption(PasswordField::PasswordField::NotRequired);
     }
 }
 
@@ -275,25 +293,31 @@ NetworkManager::VpnSetting::Ptr OpenVpnAdvancedWidget::setting() const
     NMStringMap secretData;
 
     // optional settings
-    if ( m_ui->sbCustomPort->value() > 0 ) {
+    if (m_ui->sbCustomPort->value() > 0 ) {
         data.insert(QLatin1String(NM_OPENVPN_KEY_PORT), QString::number(m_ui->sbCustomPort->value()));
     }
-    if ( m_ui->sbMtu->value() > 0 ) {
+    if (m_ui->sbMtu->value() > 0 ) {
         data.insert(QLatin1String(NM_OPENVPN_KEY_TUNNEL_MTU), QString::number(m_ui->sbMtu->value()));
     }
-    if ( m_ui->sbUdpFragmentSize->value() > 0 ) {
+    if (m_ui->sbUdpFragmentSize->value() > 0 ) {
         data.insert(QLatin1String(NM_OPENVPN_KEY_FRAGMENT_SIZE), QString::number(m_ui->sbUdpFragmentSize->value()));
     }
     if (m_ui->chkUseCustomReneg->isChecked()) {
         data.insert(QLatin1String(NM_OPENVPN_KEY_RENEG_SECONDS), QString::number(m_ui->sbCustomReneg->value()));
     }
-    data.insert( QLatin1String(NM_OPENVPN_KEY_PROTO_TCP), m_ui->chkUseTCP->isChecked() ? "yes" : "no" );
-    data.insert( QLatin1String(NM_OPENVPN_KEY_COMP_LZO), m_ui->chkUseLZO->isChecked() ? "yes" : "no" );
-    data.insert( QLatin1String(NM_OPENVPN_KEY_TAP_DEV), m_ui->chkUseTAP->isChecked() ? "yes" : "no" );
-    data.insert( QLatin1String(NM_OPENVPN_KEY_MSSFIX), m_ui->chkMssRestrict->isChecked() ? "yes" : "no" );
+    data.insert(QLatin1String(NM_OPENVPN_KEY_PROTO_TCP), m_ui->chkUseTCP->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
+    data.insert(QLatin1String(NM_OPENVPN_KEY_COMP_LZO), m_ui->chkUseLZO->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
+    if (m_ui->chkUseVirtualDeviceType->isChecked()) {
+        data.insert(QLatin1String(NM_OPENVPN_KEY_DEV_TYPE), m_ui->cmbDeviceType->currentIndex() == 0 ? QLatin1String("tun") : QLatin1String("tap"));
+    }
+    if (m_ui->chkUseVirtualDeviceName->isChecked()) {
+        data.insert(QLatin1String(NM_OPENVPN_KEY_DEV), m_ui->leVirtualDeviceName->text());
+    }
+    data.insert(QLatin1String(NM_OPENVPN_KEY_MSSFIX), m_ui->chkMssRestrict->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
+    data.insert(QLatin1String(NM_OPENVPN_KEY_REMOTE_RANDOM), m_ui->chkRandRemHosts->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
 
     // Optional Security
-    switch ( m_ui->cboHmac->currentIndex()) {
+    switch (m_ui->cboHmac->currentIndex()) {
     case Private::EnumHashingAlgorithms::Default:
         break;
     case Private::EnumHashingAlgorithms::None:
@@ -324,6 +348,11 @@ NetworkManager::VpnSetting::Ptr OpenVpnAdvancedWidget::setting() const
         data.insert(QLatin1String(NM_OPENVPN_KEY_AUTH), QLatin1String(NM_OPENVPN_AUTH_RIPEMD160));
         break;
     }
+
+    if (m_ui->chkUseCustomCipherKey->isChecked()) {
+        data.insert(QLatin1String(NM_OPENVPN_KEY_KEYSIZE), QString::number(m_ui->sbCustomCipherKey->value()));
+    }
+
     if (m_ui->cboCipher->currentIndex() != 0) {
         data.insert(QLatin1String(NM_OPENVPN_KEY_CIPHER), m_ui->cboCipher->currentText());
     }
@@ -356,21 +385,21 @@ NetworkManager::VpnSetting::Ptr OpenVpnAdvancedWidget::setting() const
     case Private::EnumProxyType::NotRequired:
         break;
     case Private::EnumProxyType::HTTP:
-        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE), "http");
+        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE), QLatin1String("http"));
         data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_SERVER), m_ui->proxyServerAddress->text());
         data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_PORT), QString::number(m_ui->sbProxyPort->value()));
-        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_RETRY), m_ui->chkProxyRetry->isChecked() ? "yes" : "no");
+        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_RETRY), m_ui->chkProxyRetry->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
         if (!m_ui->proxyUsername->text().isEmpty()) {
             data.insert(QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_USERNAME), m_ui->proxyUsername->text());
             secretData.insert(QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD), m_ui->proxyPassword->text());
-            handleOnePasswordType(m_ui->proxyPasswordStorage, QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"), data);
+            handleOnePasswordType(m_ui->proxyPassword, QLatin1String(NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD"-flags"), data);
         }
         break;
     case Private::EnumProxyType::SOCKS:
-        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE), "socks");
+        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_TYPE), QLatin1String("socks"));
         data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_SERVER), m_ui->proxyServerAddress->text());
         data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_PORT), QString::number(m_ui->sbProxyPort->value()));
-        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_RETRY), m_ui->chkProxyRetry->isChecked() ? "yes" : "no");
+        data.insert(QLatin1String(NM_OPENVPN_KEY_PROXY_RETRY), m_ui->chkProxyRetry->isChecked() ? QLatin1String("yes") : QLatin1String("no"));
         break;
     }
 
@@ -380,20 +409,6 @@ NetworkManager::VpnSetting::Ptr OpenVpnAdvancedWidget::setting() const
     return d->setting;
 }
 
-void OpenVpnAdvancedWidget::proxyPasswordStorageChanged(int index)
-{
-    setPasswordType(m_ui->proxyPassword, index);
-}
-
-void OpenVpnAdvancedWidget::proxyPasswordToggled(bool show)
-{
-    if (show) {
-        m_ui->proxyPassword->setEchoMode(QLineEdit::Normal);
-    } else {
-        m_ui->proxyPassword->setEchoMode(QLineEdit::Password);
-    }
-}
-
 void OpenVpnAdvancedWidget::proxyTypeChanged(int type)
 {
     switch (type) {
@@ -403,17 +418,13 @@ void OpenVpnAdvancedWidget::proxyTypeChanged(int type)
         m_ui->chkProxyRetry->setEnabled(false);
         m_ui->proxyUsername->setEnabled(false);
         m_ui->proxyPassword->setEnabled(false);
-        m_ui->proxyPasswordStorage->setEnabled(false);
-        m_ui->chkProxyShowPassword->setEnabled(false);
         break;
     case Private::EnumProxyType::HTTP:
         m_ui->proxyServerAddress->setEnabled(true);
         m_ui->sbProxyPort->setEnabled(true);
         m_ui->chkProxyRetry->setEnabled(true);
         m_ui->proxyUsername->setEnabled(true);
-        m_ui->proxyPasswordStorage->setEnabled(true);
-        setPasswordType(m_ui->proxyPassword, m_ui->proxyPasswordStorage->currentIndex());
-        m_ui->chkProxyShowPassword->setEnabled(true);
+        m_ui->proxyPassword->setEnabled(true);
         break;
     case Private::EnumProxyType::SOCKS:
         m_ui->proxyServerAddress->setEnabled(true);
@@ -421,25 +432,25 @@ void OpenVpnAdvancedWidget::proxyTypeChanged(int type)
         m_ui->chkProxyRetry->setEnabled(true);
         m_ui->proxyUsername->setEnabled(false);
         m_ui->proxyPassword->setEnabled(false);
-        m_ui->proxyPasswordStorage->setEnabled(false);
-        m_ui->chkProxyShowPassword->setEnabled(false);
         break;
     }
 }
 
-uint OpenVpnAdvancedWidget::handleOnePasswordType(const QComboBox * combo, const QString & key, NMStringMap & data) const
+void OpenVpnAdvancedWidget::handleOnePasswordType(const PasswordField *passwordField, const QString & key, NMStringMap & data) const
 {
-    uint type = combo->currentIndex();
-    switch (type) {
-    case SettingWidget::EnumPasswordStorageType::AlwaysAsk:
-        data.insert(key, QString::number(NetworkManager::Setting::NotSaved));
+    const PasswordField::PasswordOption option = passwordField->passwordOption();
+    switch (option) {
+    case PasswordField::StoreForAllUsers:
+        data.insert(key, QString::number(NetworkManager::Setting::None));
         break;
-    case SettingWidget::EnumPasswordStorageType::Store:
+    case PasswordField::StoreForUser:
         data.insert(key, QString::number(NetworkManager::Setting::AgentOwned));
         break;
-    case SettingWidget::EnumPasswordStorageType::NotRequired:
+    case PasswordField::AlwaysAsk:
+        data.insert(key, QString::number(NetworkManager::Setting::NotSaved));
+        break;
+    case PasswordField::NotRequired:
         data.insert(key, QString::number(NetworkManager::Setting::NotRequired));
         break;
     }
-    return type;
 }
